#
# $Log: not supported by cvs2svn $
# Revision 1.54  2006/01/31 22:03:31  soliday
# Modified APSExecLogCallback so that it does not attempt to resize the
# text widget if the text lines are longer than 150 characters. This was
# creating an extreemly wide window and the scroll bar on the right was
# getting lost of the right of the screen.
#
# Revision 1.53  2006/01/17 23:01:34  soliday
# Modified APSExecLogCallback procedure to check to ensure the text widget
# associated with the APSExecLog command still exists before trying to
# read data from it.
#
# Revision 1.52  2006/01/17 21:08:13  jiaox
# Made the command button output window  auto scale if the actual line length is large than the default  window width.
#
# Revision 1.51  2005/10/13 19:04:57  borland
# Dislabed APSRenameExecToAPSBGExec again due to continued performance issues.
#
# Revision 1.50  2005/10/12 22:46:02  borland
# Increased the time between bitmap swaps for APSBGExec to greatly reduce
# performance hit.
#
# Revision 1.49  2005/10/12 00:13:40  borland
# Disabled renaming of exec to APSBGExec because of performance problems.
#
# Revision 1.48  2005/09/19 15:28:02  soliday
# APSBGExec no longer use bgexec when piping into a file because Tcl/Tk
# occationally crashes when you do this.
#
# Revision 1.47  2005/08/23 20:04:18  soliday
# Changed APSBGExec to use normal exec when the command already has an ampersand.
#
# Revision 1.46  2005/08/23 19:04:29  soliday
# Fixed another problem returning error string from bgexec under some
# conditions.
#
# Revision 1.45  2005/08/23 18:33:35  soliday
# Fixed help message.
#
# Revision 1.44  2005/08/22 19:46:55  soliday
# Added the APSRenameExecToAPSBGExec procedure which can be used to seemlessly
# replace all exec commands with APSBGExec. This procedure would just need to be
# run once at the beginning of a program. In the future it may be called by
# APSStandardSetup if APSBGExec proves robust enough.
#
# Revision 1.43  2005/08/22 19:14:40  soliday
# Improved error message in APSBGExec when the executable is killed by the
# user. Also updated to work without a GUI and without BLT if it is not available.
#
# Revision 1.42  2005/08/22 16:47:06  soliday
# Fixed intermitent problem with APSBusyIndicatorContinue
#
# Revision 1.41  2005/08/19 22:53:32  soliday
# Updated to work on systems were the image files may not exist. In this
# case it will simply run as it did before.
#
# Revision 1.40  2005/08/19 22:12:14  soliday
# Moved the busy indicator from the scrolled text to the menu bar. I also renamed
# some of the related routines and put them in different files.
#
# Revision 1.39  2005/08/19 19:16:46  soliday
# Fixed help description for APSBGExec.
#
# Revision 1.38  2005/08/19 19:14:49  soliday
# Added APSBGExec. This can be used as a replacement to exec with the added
# feature of a busy indicator in the APSScrolledStatus screen. It also allows
# the user to kill the exec'ed program.
#
# Revision 1.37  2005/02/21 04:54:31  borland
# Added catch around wm title command in case the widget isn't a top-level widget.
#
# Revision 1.36  2004/08/19 21:42:44  shang
# added closeInput argument to APSExecLog proc
#
# Revision 1.35  2004/08/17 05:28:46  borland
# Added -killSignal option to APSExecLog.  It allows killing more effectively in
# cases where the program may trap errors.
#
# Revision 1.34  2004/06/08 15:52:45  emery
# Indented code.
#
# Revision 1.33  2003/11/03 16:34:28  soliday
# Added email button to APSExecLog procedure.
#
# Revision 1.32  2003/06/24 20:52:04  shang
# added pipePid to the return list of APSExecLog
#
# Revision 1.31  2001/05/18 21:59:01  soliday
# Fixed a problem with the usage message.
#
# Revision 1.30  2001/03/15 21:55:40  emery
# Fixed typo in usage message.
#
# Revision 1.29  2001/02/06 19:44:22  soliday
# Fixed some tcl warning messages found with procheck
#
# Revision 1.28  2000/11/27 21:02:58  borland
# APSExecLog now accepts -font option.
#
# Revision 1.27  1999/06/14 23:50:35  borland
# Added some catch statements to APSExecLog so that if the window
# is destroyed before the widgets are all configured, there is no
# error displayed.
#
# Revision 1.26  1998/10/21 22:01:24  borland
# APSExecLog now returns the file ID, which can be used with APSExecLogAbort.
#
# Revision 1.25  1998/06/16 21:07:40  borland
# Added more catch statements to APSExecLogCallback to prevent tcl errors
# when the process is killed while the log window is still open.
#
# Revision 1.24  1998/06/16 20:09:25  borland
# Added catch around window-destroying command in APSExecLogAbort.
#
# Revision 1.23  1998/06/16 16:48:34  borland
# Fixed bug in APSExecLogAbort.  Needed to eval exec kill instead of
# exec kill, otherwise didn't reliably kill processes (list of PVs vs
# single PV).
#
# Revision 1.22  1997/04/25 19:30:26  borland
# Added -lineLimit option to APSExecLog, so that the size of the scroll region
# can be limited.
#
# Revision 1.21  1996/10/09 19:08:00  saunders
# Improved abort behavior if process already completed.
#
# Revision 1.20  1996/10/09 18:05:58  saunders
# Added public procedure APSExecLogAbort. APSExecLog now returns an
# id which may subsequently be used as arg to APSExecLogAbort.
#
# Revision 1.19  1996/10/04 19:53:02  saunders
# Added "print", "save as..", and "email..." buttons to APSAlertBox.
# In doing so, I removed functionally duplicate procedures, and created
# generic APSPrint and APSSaveDialog. Also addes "save as.." to APSExecLog.
# Removed "Enscript" buttons, since "Print" is adequate.
#
# Revision 1.18  1996/06/25 21:13:49  saunders
# Commands which produce a Ctrl-G (007) will now generate a beep. The
# char is stripped out of the output before being displayed.
#
# Revision 1.17  1996/05/14 18:37:39  borland
# Added -cancelCallback and -abortCallback options and implementation.
#
# Revision 1.16  1996/02/07  23:23:27  saunders
# Restricted form of -unixCommand to permit process aborting.
#
# Revision 1.15  1996/01/31  18:59:42  saunders
# Callback now performed at global level.
#
# Revision 1.14  1996/01/23  20:51:19  saunders
# Improved scroll behavior added by Ken Evans. Autoraise disabled.
#
# Revision 1.13  1996/01/16  23:40:50  saunders
# Further refinement of scrolling behavior.
#
# Revision 1.12  1996/01/16  23:13:36  saunders
# Window now forces scroll to bottom when new command is used on
# an existing exec log window.
#
# Revision 1.11  1996/01/16  21:20:00  saunders
# Added Log for CVS. Should have done this a long time ago.
#
#

proc APSExecLogHelp {} {
    return "Usage: APSExecLog widget \n\[-name <string>\]\t\n\[-parent <string>\]\t\n\[-noPack 1\]\t\n\[-packOption <list>\]\t\n\[-unixCommand <string>\]\t\n\[-callback <script>\]\t\n\[-abortCallback <script>\]\t\n\[-cancelCallback <script>\]\t\n\[-width <string>\]\t\n\[-height <string>\]\t\n\[-contextHelp <string>\]\t\n\[-lineLimit <number>\]\t\n\nCreates\t\$parent\$widget.\n\tuserFrame\n\tbuttonRow.ok.button\n\tbuttonRow.cancel.button\n\tbuttonRow.print.button\n\tuserFrame.text.text\n\tuserFrame.text.scroll\n\nReturns an <id> for subsequent use in APSExecLogAbort procedure, or -1 if open failed."
}

set apsExecLogID 0
proc APSExecLog {widget args} {
    global apsContextHelp
    set parent ""
    set noPack 0
    set packOption "-side top -fill x"
    set unixCommand ""
    set contextHelp ""
    set callback ""
    set abortCallback ""
    set cancelCallback ""
    set name "Unix Command Execution"
    set width 40
    set height 15
    set lineLimit 0
    set font ""
    set killSignal 15
    set closeInput 1
    APSStrictParseArguments {parent unixCommand noPack packOption contextHelp \
                               callback name width height abortCallback cancelCallback  \
                               lineLimit font killSignal closeInput}

    set windowExists [winfo exists $parent$widget]

    if {!$windowExists} {
        APSDialogBox $widget -parent $parent -name $name \
          -noPack $noPack -packOption $packOption \
          -contextHelp $contextHelp
        
        APSScrolledText .text -parent $parent$widget.userFrame -width $width \
          -height $height -font $font
    } else {
        catch {wm title $parent$widget $name}
    }

    set log $parent$widget.userFrame.text.text

    if {!$windowExists} {
        # Put in a null command until the process pipe is open
        APSDialogBoxAddButton .abort -parent $parent$widget -text Abort \
          -command "" \
          -contextHelp "Abort command execution, but leave window up."
        
        APSDialogBoxAddButton .print -parent $parent$widget -text Print \
          -command "APSPrint -textWidget $log -cmd enscript" \
          -contextHelp "Print contents of window using enscript command."

        APSDialogBoxAddButton .email -parent $parent$widget -text "Email..." \
          -command "APSScrolledTextEMail -textWidget $log -modal 1" \
          -contextHelp "Send the contents of window as an email message."

        APSButton .save -parent $parent$widget.buttonRow -text "Save As..."\
          -command "APSSaveDialog .alertSave -textWidget $log" \
          -contextHelp "Save contents of window to a file."
    } else {
        APSDisableButton $parent$widget.buttonRow.ok.button
    }

    global apsExecLogLineCount  apsExecLogLineLimit
    set apsExecLogLineCount($log) 0
    set apsExecLogLineLimit($log) $lineLimit
    
    #    set cmd1 "csh -f -c \"$unixCommand\" |& cat -u"
    set cmd1 "$unixCommand |& cat -u"
    # If at end now, then reset to end afterward
    if {[lindex [$log yview] 1] == 1} {
        set gotoend 1
    } else {
        set gotoend 0
    }
    if {[catch {open |$cmd1 r} input]} {
        $log insert end $input\n
        if {$gotoend} {$log see end}
        set pipePid -1
        return -1
    } else {
        set pipePid [pid $input]
        fileevent $input readable \
          [list APSExecLogCallback $parent$widget $input $log $callback]
        $log insert end $unixCommand\n
        if {$gotoend} {$log see end}
    }
    catch {$parent$widget.buttonRow.cancel.button configure -command \
             [list APSExecLogStop $parent$widget $input $pipePid $cancelCallback $killSignal]}
    catch {$parent$widget.buttonRow.abort.button configure \
             -command [list APSExecLogAbortInternal $input $pipePid $abortCallback $killSignal $closeInput]}
    return [list $input $parent$widget $pipePid]
}

#
# Public procedure which allows user to Abort the unixCommand, and
#  optionally destroy the ExecLog widget.
#

proc APSExecLogAbortHelp {} {
    return "Usage: APSExecLogAbort \[-id <id>\] \[-destroy 1\]"
}

proc APSExecLogAbort {args} {
    set id -1
    set destroy 0
    APSStrictParseArguments {id destroy}

    set input [lindex $id 0]
    if {[catch {pid $input} pipePid]} {
        set kill 0
    } else {
        set kill 1
    }

    if {$kill} {
        catch {eval exec kill $pipePid}
        catch {close $input}
    }
    if {$destroy} {
        set widget [lindex $id 1]
        catch {destroy $widget}
    }
}

proc APSExecLogCallback {widget input log callback} {
    if {[eof $input]} {
        catch {close $input}
        catch {APSEnableButton $widget.buttonRow.ok.button}
        if {[winfo exists $log]} {
            set output [ $log get 0.0 end]
            set output [ split $output "\n"]
            set linewidth ""
            foreach line $output {
                lappend linewidth [string length $line]
            }
            set linewidth [ lsort -integer -decreasing $linewidth ]
            set w 0
            foreach width $linewidth {
                if {$width < 150} {
                    set w $width
                    break
                }
            }
            if { $w > [ $log cget -width] } {
                $log  configure -width  [expr $w+5 ]
            }
        }
        # used to be 'eval callback', really want it done at global level
        uplevel #0 $callback
    } else {
        # Add new output, but don't disturb view if user has scrolled up
        set oldend 0
        if {![catch {$log yview} view]} {
            set oldend [lindex $view 1]
        } 
        gets $input line
        # Scan for any bell chars. Strip out the char, and do a tk bell.
        set ix [string first "\007" $line]
        while {$ix != -1} {
            bell
            set line "[string range $line 0 [expr {$ix - 1}]][string range $line [expr {$ix + 1}] end]"
            set ix [string first "\007" $line]
        }

        catch {$log insert end $line\n}
        if {$oldend == 1} {
            catch {$log see end}
        }
        global apsExecLogLineCount  apsExecLogLineLimit
        incr apsExecLogLineCount($log)
        if {$apsExecLogLineLimit($log) && \
              $apsExecLogLineCount($log)>$apsExecLogLineLimit($log)} {
            catch {$log delete 1.0 2.0}
            incr apsExecLogLineCount($log) -1
        }
    }
}

proc APSExecLogStop {widget input pipePid callback signal} {
    catch "exec kill -$signal $pipePid" 
    catch {close $input}
    destroy $widget
    if {[string length $callback]} {
        uplevel \#0 $callback
    }
}

proc APSExecLogAbortInternal {input pipePid callback signal closeInput} {
    if $closeInput {
        catch "exec kill -$signal $pipePid"
        catch {close $input}
    } else {
        catch "exec kill -$signal [lindex $pipePid 0]"
    }
    if {[string length $callback]} {
        uplevel \#0 $callback
    }
}

# Replaced by APSPrint. This kept for compatibility only.
proc APSExecLogPrint {textwidget cmd} {
    APSPrint -textWidget $textwidget -cmd $cmd
}

proc APSExecHelp {} {
    return "Usage: APSExec\n\[-unixCommand <string>\]\t\n\[-callback <script>\]\t\n\[-outputVariable <variable>\]\t\n\nExecutes command given by -unixCommand option.\nIf -callback is given, <script> will be executed upon\ncompletion of unixCommand. Output of unixCommand\nis normally thrown away. If -outputVariable is given,\ncommand output will be stored in global <variable>."
}
proc APSExec {args} {
    set unixCommand ""
    set callback ""
    set outputVariable ""
    APSStrictParseArguments {unixCommand callback outputVariable}

    if {$outputVariable != ""} {
        set saveOutput 1
        global $outputVariable
    } else {
        set saveOutput 0
    }
    #    set cmd1 "csh -f -c \"$unixCommand\" |& cat"
    set cmd1 "$unixCommand |& cat -u"
    if {[catch {open |$cmd1 r} input]} {
        if {$saveOutput} {
            set $outputVariable $input
        }
    } else {
        fileevent $input readable \
          [list APSExecCallback $input $callback $outputVariable $saveOutput]
        set $outputVariable ""
        return $input
    }
}

proc APSExecCallback {input callback outputVariable saveOutput} {
    if {$saveOutput} {
        global $outputVariable
    }
    if {[eof $input]} {
        catch {close $input}
        uplevel #0 $callback
        # used to be 'eval $callback', really want this done at global level
    } else {
        gets $input line
        if {$saveOutput} {
            set $outputVariable [subst $$outputVariable]$line\n
        }
    }
}

proc APSBGExecHelp {} {
    return "Usage: APSBGExec\n\[-command <string>\]\t\n\[-output <variable>\]\t\n\[-error <variable>\]\t\n\[-status <variable>\]\t\n\nExecutes command given by -command option.\nWorks like the normal exec command where you can catch the results and errors. If the -output, -error, and or status variables are used the related information will be stored in those global variables. One advantage of using APSBGExec over exec is that there will be a busy indicator in the menu bar and the executable is killable if necessary."
}

proc APSBGExec {args} {
    set output "apsBGExecOutput"
    set error "apsBGExecError"
    set status "apsBGExecStatus"
    set command ""
    APSStrictParseArguments {output error status command}
    global $error $output $status apsBGExecKill apsExecRenamed
    set blt 1
    if {[catch {package require BLT} result]} {
        set blt 0
    }
    set $output ""
    set $error ""
    set $status ""

    set apsBGExecKill 0
    if {([llength [info commands winfo]]) && ([winfo exists .menu.working])} {
        APSBusyIndicatorStart -killVariable $status
    }
    if {([string index [string trimright $command] end] == "&") ||
        ([string first ">" $command] != -1)} {
        set blt 0
    }
    if {$blt} {
        if {[catch {eval blt::bgexec $status -output $output -error $error $command} result]} {
            if {([llength [info commands winfo]]) && ([winfo exists .menu.working])} {
                APSBusyIndicatorStop
            }
            if {$apsBGExecKill} {
                return -code error "APSBGExec command killed by user."
            }
            if {![llength [set $error]]} {
                set $error $result
            }
            return -code error "[set $error]"
        }
    } else {
        if {$apsExecRenamed} {
            set execcommand aps_tcl_exec
        } else {
            set execcommand exec
        }
        if {[catch {eval $execcommand $command} result]} {
            if {([llength [info commands winfo]]) && ([winfo exists .menu.working])} {
                APSBusyIndicatorStop
            }
            set $error $result
            set $status ""
            return -code error "[set $error]"
        } else {
            if {([llength [info commands winfo]]) && ([winfo exists .menu.working])} {
                APSBusyIndicatorStop
            }
            set $output $result
            set $status ""
            return -code ok "[set $output]"
        }
    }
    if {$apsBGExecKill} {
        if {([llength [info commands winfo]]) && ([winfo exists .menu.working])} {
            APSBusyIndicatorStop
        }
        return -code error "APSBGExec command killed by user."
    }
    if {([llength [info commands winfo]]) && ([winfo exists .menu.working])} {
        APSBusyIndicatorStop
    }
    return -code ok "[set $output]"
}

proc APSBusyIndicatorStartHelp {} {
    return "Usage: APSBusyIndicatorStart \n\[-killVariable <variable>\]\t\n\nThis procedure is used to start the busy indicator ont he menu bar. If the killVariable option is passed it is assumed that the bgexec command is being called and that setting this variable will kill the bgexec command. This is done by the user pressing the stop sign button."
}

proc APSBusyIndicatorStart { args } {
    if {![llength [info commands winfo]]} {
        return
    }
    if {![winfo exists .menu.working]} {
        return
    }
    set killVariable ""
    APSStrictParseArguments {killVariable}
    global apsBusyIndicatorWorking apsBGExecKill
    set apsBusyIndicatorWorking 1
    .menu.working configure -image working1
    set apsBGExecKill 0
    if {[llength $killVariable]} {
        .menu.stop configure -image stopsign
        .menu.stop configure  -command "set apsBGExecKill 1 ; set $killVariable 1"
    } else {
        .menu.stop configure  -command ""
    }
    update
    after 100 "APSBusyIndicatorContinue"
}

proc APSBusyIndicatorContinue { args } {
    global apsBusyIndicatorWorking
    if {$apsBusyIndicatorWorking} {
        set i [string index [.menu.working cget -image] end]
        if {($i < 1) || ($i > 4)} { set i 1 }
        incr i
        if {$i > 4} {set i 1}
        .menu.working configure -image working$i
        after 2000 "APSBusyIndicatorContinue"
    }
}

proc APSBusyIndicatorStopHelp {} {
    return "Usage: APSBusyIndicatorStop\t\n\nUsed with APSBusyIndicatorStart to stop the busy indicator."
}
proc APSBusyIndicatorStop { args } {
    if {![llength [info commands winfo]]} {
        return
    }
    if {![winfo exists .menu.working]} {
        return
    }
    global apsBusyIndicatorWorking
    APSSetVarAndUpdate apsBusyIndicatorWorking 0
    .menu.stop configure -image nostopsign
    .menu.working configure -image working
}

proc APSExecReplacement {args} {
    if [catch {APSBGExec -command "$args"} result] {
        return -code error "$result"
    }
    return -code ok "$result"
}

proc APSRenameExecToAPSBGExecHelp {} {
    return "Usage: APSRenameExectoAPSBGExec\t\n\nAfter this is called every exec command is actually using APSBGExec. The original exec command will be renamed to aps_tcl_exec."
}

set apsExecRenamed 0
proc APSRenameExecToAPSBGExec {args} {
    global apsExecRenamed

    # Disable due to performance problems.
    return 

    if {$apsExecRenamed == 0} {
        set apsExecRenamed 1
        rename exec aps_tcl_exec
        rename APSExecReplacement exec
    }
}
