
set apsSRExcitationFileDirectory /home/helios/oagData/sr/magnets
set apsBTSExcitationFileDirectory /home/helios/oagData/bts/magnets

proc APSLatticeToSetpointsQuadSext {args} {
    global apsSRExcitationFileDirectory 
    set input ""
    set output ""
    set includeQuadrupoles 1
    set includeSextupoles  1
    set energyGeV 6.0
    APSStrictParseArguments {input output includeQuadrupoles includeSextupoles energyGeV}
    if ![string length $input] {
        return -code error "APSLatticeToSetpointsQuadSext: provide input filename"
    }
    if ![string length $output] {
        return -code error "APSLatticeToSetpointsQuadSext: provide output filename"
    }
    if ![file exists $input] {
        return -code error "APSLatticeToSetpointsQuadSext: not found: $input"
    }
    if [file exists $output] {
        return -code error "APSLatticeToSetpointsQuadSext: in use: $output"
    }

    set typeList {QUAD SEXT}
    set BnLList {B1L B2L}
    set KnList {K1 K2}
    set summaryFileList {QuadSummary.sdds SextSummary.sdds}
    set includeList [list $includeQuadrupoles $includeSextupoles]
    set resultList ""

    exec sddsconvert $input $input.bare -delete=param,* -retain=column,ElementName,ElementType,ElementParameter,ParameterValue
    APSAddToTmpFileList -ID APSLatticeToSetpointsQuadSext -fileList $input.bare
    
    set typesDone 0
    foreach type $typeList BnL $BnLList Kn $KnList summaryFile $summaryFileList include $includeList {
        if !$include continue
        if [catch {exec sddsprocess $input.bare -pipe=out -match=col,ElementType=*${type} -nowarning \
                       | sdds2stream -pipe -rows=bare} count] {
            return -code error "APSLatticeToSetpointsQuadSext (1): $count"
        }
        if $count==0 continue
        #puts stderr "$type $count"
        
        # Get the fractional strength errors
        APSAddToTmpFileList -ID APSLatticeToSetpointsQuadSext -fileList $input.FSE
        if [catch {exec sddsprocess $input.bare $input.FSE -match=col,ElementType=*${type} \
                       -match=col,ElementParameter=FSE -nowarning} result] {
            return -code error "APSLatticeToSetpointsQuadSext (2): $result"
        }
        #puts stderr [exec sddsprintout -col $input.FSE]

        # Get the Kn values
        if [catch {exec sddsprocess $input.bare -pipe=out -match=col,ElementType=*${type} -match=col,ElementParameter=${Kn} -nowarning \
                       = Combine Kn values with the lengths and FSEs = \
                       | sddsxref $input.FSE -pipe -take=ParameterValue -rename=col,ParameterValue=FSE \
                       | sddsprocess -pipe -edit=column,MagnetTypeName,ElementName,Z: -nowarning \
                       | sddsxref ${apsSRExcitationFileDirectory}/$summaryFile -pipe -reuse=row \
                       -match=MagnetTypeName -take=Length,KnSign \
                       | sddsprocess -pipe \
                       "-define=parameter,E,$energyGeV,units=GeV" \
                       "-define=parameter,H,E 1e9 * c_mks /,units=T*m" \
                       "-define=col,Kn,ParameterValue,units=SI" \
                       "-define=col,KnLDesired,Kn Length * 1 FSE + *,units=SI" \
                       "-define=col,BnLDesired,KnLDesired H * KnSign *,units=SI" \
                       -print=column,Filename,${apsSRExcitationFileDirectory}/%sMain/%s.excit,MagnetTypeName,ElementName \
                       -print=column,ControlName,%s:PS:SetCurrentC,ElementName \
                       | sddsinterpset -pipe \
                       -order=1 -data=fileColumn=Filename,interpolate=Current,functionOf=${BnL},column=BnLDesired \
                       | sddsprocess -pipe=in $input.${type} -nowarning \
                       "-print=col,ValueString,%21.15le,Current"} result] {
            return -code error "APSLatticeToSetpointsQuadSext (3): $result"
        }
        APSAddToTmpFileList -ID APSLatticeToSetpointsQuadSext -fileList $input.$type
        
        file delete $input.FSE
        lappend resultList $input.${type}
        incr typesDone 1
    }
    
    if { $typesDone==0 } {
        # Not necessarily an error
        APSDeleteTmpFileList -ID APSLatticeToSetpointsQuadSext 
       return $typesDone
    }
    eval exec sddscombine $resultList -merge $output -retain=col,ControlName,ValueString,Current,Element*,Parameter*
    APSDeleteTmpFileList -ID APSLatticeToSetpointsQuadSext
    return $typesDone
}

proc APSSetpointsToLatticeQuadSext {args} {
    global apsSRExcitationFileDirectory
    set input ""
    set output ""
    set energyGeV 6.0
    APSStrictParseArguments {input output energyGeV}
    if ![string length $input] {
        return -code error "APSSetpointsToLatticeQuadSext: provide input filename"
    }
    if ![string length $output] {
        return -code error "APSSetpointsToLatticeQuadSext: provide output filename"
    }
    if ![file exists $input] {
        return -code error "APSSetpointsToLatticeQuadSext: not found: $input"
    }
    if [file exists $output] {
        return -code error "APSSetpointsToLatticeQuadSext: in use: $output"
    }

    set typeList {KQUAD KSEXT}
    set wildcardList {S??\[AB\]:Q\[12367\] S??\[AB\]:S\[123\]}
    set summaryFileList {QuadSummary.sdds SextSummary.sdds}
    set multList {1 2}

    set resultList ""
    
    foreach type $typeList wildcard $wildcardList mult $multList summaryFile $summaryFileList {
        #APSSetVarAndUpdate status "APSSetpointsToLatticeQuadSext $type"
        # Need only the PV name and value
        APSAddToTmpFileList -ID APSSetpointsToLatticeQuadSext -fileList $output.$type.tmp1
        exec sddsconvert $input -pipe=out -retain=col,ControlName,ValueString \
            = Select only the PV names corresponding to the type of magnets = \
            | sddsprocess -pipe=in $output.$type.tmp1  "-match=col,ControlName=${wildcard}:PS:SetCurrentC" -nowarning \
            = Create the element name (used in the lattice) and scan the current = \
            -edit=column,ElementName,ControlName,2S/:/100D -scan=column,Current,ValueString,%le \
            "-edit=col,MagnetTypeName,ElementName,Z:" \
            = Create the name of the excitation file = \
            -print=column,Filename,${apsSRExcitationFileDirectory}/%sMain/%s.excit,MagnetTypeName,ElementName 
            
        if [catch {exec sdds2stream -rows=bare $output.$type.tmp1} count] {
            return -code error "APSSetpointsToLatticeQuadSext: $count"
        }
        if { $count != 0} {
            #APSSetVarAndUpdate status "APSSetpointsToLatticeQuadSext $type $count"
            # Get length, sign for each element 
            APSAddToTmpFileList -ID APSSetpointsToLatticeQuadSext -fileList $output.${type}
            exec sddsxref $output.$type.tmp1 ${apsSRExcitationFileDirectory}/$summaryFile -pipe=out -reuse=row -nowarning \
                -match=MagnetTypeName -take=Length,KnSign \
                | sddsinterpset -pipe \
                = Interpolate all the files at the current values found in the input file = \
                -order=1 -data=fileColumn=Filename,interpolate=B${mult}L,functionOf=Current,column=Current \
                | sddsconvert -pipe -rename=col,B${mult}L=BnL \
                | sddsprocess -pipe=in $output.${type} -nowarning \
                = Convert BnL to Kn = \
                "-redefine=parameter,E,$energyGeV,units=GeV" \
                "-redefine=parameter,H,E 1e9 * c_mks /,units=T*m" \
                "-redefine=col,Bn,BnL KnSign * Length /,units=SI" \
                "-redefine=col,n,${mult},type=short" \
                "-redefine=col,ParameterValue,Bn H /,units=SI" \
                "-reprint=col,ElementParameter,K${mult}" \
                -reprint=col,ElementType,$type
            
            # Also set the FSE values to 0 in case these are set to something else in the lattice
            APSAddToTmpFileList -ID APSSetpointsToLatticeQuadSext -fileList $output.${type}.FSE
            exec sddsconvert $output.${type} -pipe=out -retain=col,ElementName \
                | sddsprocess -pipe=in $output.${type}.FSE -nowarning \
                -reprint=col,ElementParameter,FSE -redefine=col,ParameterValue,0 \
                -reprint=col,ElementType,$type
            
            lappend resultList $output.$type $output.$type.FSE
        }
        file delete $output.$type.tmp1
    }
    #APSSetVarAndUpdate status "$resultList"
    if [llength $resultList]==0 {
        APSDeleteTmpFileList -ID APSSetpointsToLatticeQuadSext
        return 0
    }
    eval exec sddscombine $resultList -pipe=out -merge -retain=col,Element*,Parameter* \
        | sddsprocess -pipe=in $output "-redefine=col,ElementOccurence,1,type=long"
    
    APSDeleteTmpFileList -ID APSSetpointsToLatticeQuadSext
    
    return 1
}

proc APSLatticeToSetpointsTGD {args} {
    global apsSRExcitationFileDirectory
    set input ""
    set output ""
    set type ""
    set energyGeV 6.0
    APSStrictParseArguments {input output type energyGeV}

    if ![string length $input] {
        return -code error "APSLatticeToSetpointsTGD: provide input filename"
    }
    if ![string length $output] {
        return -code error "APSLatticeToSetpointsTGD: provide output filename"
    }
    if ![file exists $input] {
        return -code error "APSLatticeToSetpointsTGD: not found: $input"
    }
    if [file exists $output] {
        return -code error "APSLatticeToSetpointsTGD: in use: $output"
    }
    
    set knownTypeList [list Q4 Q5 Q8 M3 M4]
    if ![string length $type] {
        return -code error "APSLatticeToSetpointsTGD: provide type name, one of [join $knownTypeList ,]"
    }
    if {[lsearch -exact $knownTypeList $type]==-1} {
        return -code error "APSLatticeToSetpointsTGD: unknown magnet type: $type"
    }

    # Collect relevant data into a file with one line per magnet, ensuring that the order of
    # the magnets is the same as in the excitation file that we are going to interpolate
    if [catch {exec sddsprocess $input -pipe=out -match=col,ElementType=C?BEND -match=col,ElementName=*:$type -nowarning \
                   | sdds2stream -pipe -rows=bare} count] {
        return -code error "APSLatticeToSetpointsTGD (0): $count"
    }
    if {$count==0} {
        return 0
    }

    APSAddToTmpFileList -ID APSLatticeToSetpointsTGD -fileList $input.trans
    
    if [catch {exec sddsprocess $input -pipe=out -match=col,ElementType=C?BEND -match=col,ElementName=*:$type -nowarning \
                   = Keep only the fields we need = \
                   | sddsconvert -pipe -retain=col,ElementName,ElementParameter,ParameterValue \
                   = Sort by ElementName, subsort by ElementParameter to prepare for transposition = \
                   | sddssort -pipe -col=ElementName -col=ElementParameter \
                   = Each page needs to pertain to a single element = \
                   | sddsbreak -pipe -change=ElementName \
                   = Need this so we can get the element name back after transposition = \
                   | sddsprocess -pipe -process=ElementName,first,%s0 -nowarning \
                   = Transpose data to make a column corresponding to each ElementParameter = \
                   = E.g., K1, ANGLE, L, ... = \
                   | sddstranspose -pipe -newcol=ElementParameter \
                   = The next four lines produce a table with a column for ElementName as well as desired element parameters = \
                   | sddsconvert -pipe -retain=parameter,ElementName0 -rename=param,ElementName0=ElementName \
                   | sddsexpand -pipe \
                   | sddscollapse -pipe \
                   | sddsconvert -pipe -retain=col,ElementName,ANGLE,K1,K2,L,FSE*,*KICK \
                   = Convert to B0L and B1L values, form name for excitation file = \
                   | sddsprocess -pipe=in $input.trans  -nowarning \
                   "-define=param,H,$energyGeV 1e9 * c_mks /,units=T*m" \
                   = B0L or BigB0 is always positive in the measurement files while B1L or BigB1 is always negative = \
                   "-define=col,BigB0,ANGLE XKICK + H * 1 FSE + FSE_DIPOLE + * abs,units=T*m" \
                   "-define=col,BigB1,K1 H * L * 1 FSE + FSE_QUADRUPOLE + * abs chs,units=T" \
                   "-print=col,GridFilename,${apsSRExcitationFileDirectory}/${type}Main/%s.excit,ElementName"} result] {
        return -code error "APSLatticeToSetpointsTGD (1): $result"
    }

    # Collect all the excitation files for the magnets in question (note that there may not be 40 or 80 magnets
    # in the caller's request, so we can't do this ahead of time). This ensures that the excitation files are
    # in the same order as the lines in the .trans file
    set fileList [exec sdds2stream -column=GridFilename $input.trans]
    APSAddToTmpFileList -ID APSLatticeToSetpointsTGD -fileList $input.excit
    if [catch {eval exec sddscombine [join $fileList  " "] -overwrite $input.excit} result] {
        return -code error "APSLatticeToSetpointsTGD (2): $result"
    }
    
    # Perform grid search and interpolation for each magnet
    APSAddToTmpFileList -ID APSLatticeToSetpointsTGD -fileList $input.interp
    if [catch {exec sddsfindin2dgrid $input.excit -pipe=out \
                   -gridVariables=MainCurrent,TrimCurrent -presorted \
                   -findLocationOf=BigB0,BigB1 -valuesFile=$input.trans -interpolate -mode=onePairPerPage \
                   | sddsxref -pipe=in $input.trans $input.interp -take=*} result] {
        return -code error "APSLatticeToSetpointsTGD (3): $result"
    }        
    
    # Convert to snapshot form
    # - Main current
    APSAddToTmpFileList -ID APSLatticeToSetpointsTGD -fileList [list $input.interp.main $input.interp.trim]
    if [catch {exec sddsprocess $input.interp -pipe=out \
                   -print=col,ControlName,%s:PS:SetCurrentC,ElementName \
                   -print=col,ValueString,%21.15e,MainCurrent \
                   | sddsconvert -pipe=in $input.interp.main \
                   -retain=col,MainCurrent,ElementName,MagnetQrId,ValueString,ControlName \
                   -rename=col,MainCurrent=Current} result] {
        return -code error "APSLatticeToSetpointsTGD (4): $result"
    }
    
    # - Trim current
    if [catch {exec sddsprocess $input.interp -pipe=out \
                   -print=col,ControlName,%sT:PS:SetCurrentC,ElementName \
                   -print=col,ValueString,%21.15e,TrimCurrent \
                   | sddsconvert -pipe=in $input.interp.trim \
                   -retain=col,TrimCurrent,ElementName,MagnetQrId,ValueString,ControlName \
                   -rename=col,TrimCurrent=Current} result] {
        return -code error "APSLatticeToSetpointsTGD (5): $result"
    }        
    
    # - Combined
    if [catch {exec sddscombine $input.interp.main $input.interp.trim -pipe=out -merge \
                   | sddssort -pipe=in $output -column=ElementName} result] {
        return -code error "APSLatticeToSetpointsTGD (6): $result"
    }
    
    APSDeleteTmpFileList -ID APSLatticeToSetpointsTGD
    return 1
}

proc APSSetpointsToLatticeTGD {args} {
    global apsSRExcitationFileDirectory
    set input ""
    set output ""
    set type ""
    set energyGeV 6.0
    APSStrictParseArguments {input output type energyGeV}

    if ![string length $input] {
        return -code error "APSSetpointsToLatticeTGD: provide input filename"
    }
    if ![string length $output] {
        return -code error "APSSetpointsToLatticeTGD: provide output filename"
    }
    if ![file exists $input] {
        return -code error "APSSetpointsToLatticeTGD: not found: $input"
    }
    if [file exists $output] {
        return -code error "APSSetpointsToLatticeTGD: in use: $output"
    }
    
    set knownTypeList [list Q4 Q5 Q8 M3 M4]
    set elementTypeList [list CCBEND CCBEND CSBEND CSBEND CSBEND]
    if ![string length $type] {
        return -code error "APSSetpointsToLatticeTGD: provide type name, one of [join $knownTypeList ,]"
    }
    if {[set index [lsearch -exact $knownTypeList $type]]==-1} {
        return -code error "APSSetpointsToLatticeTGD: unknown magnet type: $type"
    }

    if [catch {exec sddsprocess $input -pipe=out -match=col,ControlName=S???:${type}:PS:SetCurrentC -nowarning \
                   | sdds2stream -pipe -rows=bare} count] {
        return -code error "APSSetpointsToLatticeTGD: $count"
    }
    if {$count==0} {
        return 0
    }

    APSAddToTmpFileList -ID APSSetpointsToLatticeTGD \
        -fileList [list $input.tmp1 $input.tmp2 $input.samplePoints $input.excit $input.values $input.values.B0 \
                      $input.values.FSE $input.values.XKICK $input.values.B1 $input.values.FSEQ]
    
    # Extract main current from the file
    exec sddsconvert $input -pipe=out -retain=col,ControlName,ValueString \
      = Prepare the PV name and value for the main supply = \
      | sddsprocess -pipe=in $input.tmp1 -nowarning \
      "-match=col,ControlName=S???:${type}:PS:SetCurrentC" \
      -edit=column,ElementName,ControlName,2S/:/100D \
      -scan=column,MainCurrent,ValueString,%le
    
    # Extract trim current from the file
    exec sddsconvert $input -pipe=out -retain=col,ControlName,ValueString \
        = Prepare the PV name and value for the trim supply = \
        | sddsprocess -pipe=in $input.tmp2 -nowarning \
        "-match=col,ControlName=S???:${type}T:PS:SetCurrentC" \
        -edit=column,ElementName,ControlName,%/T:PS/:PS/a2S/:/100D \
        -scan=column,TrimCurrent,ValueString,%le
    
    # Combine main and trim setpoint pairs, one per page
    exec sddsxref $input.tmp1 $input.tmp2  -match=ElementName -pipe=out -take=TrimCurrent -nowarning \
        | sddsconvert -pipe -retain=col,TrimCurrent,MainCurrent,ElementName \
        | sddsprocess -pipe -nowarning \
        "-print=col,GridFilename,${apsSRExcitationFileDirectory}/${type}Main/%s.excit,ElementName" \
        | sddssort -pipe=in -column=ElementName $input.samplePoints
    
    file delete $input.tmp1 $input.tmp2
    
    # Collect all the excitation files for the magnets in question (note that there may not be 40 or 80 magnets
    # in the caller's request, so we can't do this ahead of time). This ensures that the excitation files are
    # in the same order as the lines in the .trans file
    set fileList [exec sdds2stream -column=GridFilename $input.samplePoints]
    eval exec sddscombine [join $fileList  " "] -overwrite $input.excit

    # Perform 2d interpolation of the grid data for magnets at the PS setpoints
    exec sddsfindin2dgrid $input.excit -pipe=out \
      = MainCurrent, TrimCurrent are the grid variables and are previously sorted for speed = \
      -gridVariables=MainCurrent,TrimCurrent -presorted \
      = BigB0, BigB1 are functions of MainCurrent, TrimCurrent = \
      -findLocationOf=BigB0,BigB1 \
      = Instead of searching/interpolating for MainCurrent, TrimCurrent that give desired values of = \
      = BigB0, BigB1, we just want to interpolate the grid at specified MainCurrent, TrimCurrent pairs = \
      -inverse -interpolate \
      = There is one sample point in the .samplePoints file per page in the .exit file = \
      -valuesFile=$input.samplePoints -mode=onePairPerPage \
      | sddsxref -pipe=in $input.samplePoints $input.values -take=ElementName
    
    # Convert the B0L deviation from nominal into a dipole fractional strength error
    # Note that B0L = BigB0 is always positive in the measurement files
    exec sddsprocess $input.values -pipe=out \
        "-define=parameter,H,$energyGeV 1e9 * c_mks /,units=T*m" \
        -edit=col,MagnetTypeName,ElementName,Z: \
        | sddsxref ${apsSRExcitationFileDirectory}/BendSummary.sdds -pipe \
        -match=MagnetTypeName -take=Angle,AngleSign -rename=col,Angle=AngleNominal -reuse=row \
        | sddsprocess -pipe=in $input.values.B0 \
        -print=column,ElementParameter,FSE_DIPOLE \
        "-define=column,ParameterValue,BigB0 H / AngleNominal / abs 1 -" \
        -print=column,ElementType,[lindex $elementTypeList $index]

    # Define FSE as 0
    exec sddsprocess $input.values.B0  $input.values.FSE \
        -reprint=col,ElementParameter,FSE -redefine=col,ParameterValue,0
    # Define XKICK as 0
    exec sddsprocess $input.values.B0  $input.values.XKICK \
        -reprint=col,ElementParameter,XKICK -redefine=col,ParameterValue,0
    
    # Convert the B1L value into a K1 value using the effective length
    # Note that B1L = BigB1 is always negative in the measurement files
    exec sddsprocess $input.values -pipe=out \
        "-define=parameter,H,$energyGeV 1e9 * c_mks /,units=T*m" \
        -edit=col,MagnetTypeName,ElementName,Z: \
        | sddsxref ${apsSRExcitationFileDirectory}/BendSummary.sdds -pipe \
        -match=MagnetTypeName -take=LEffective,K1Sign -edit=column,B1*,ei/Nominal/ -reuse=row \
        | sddsprocess -pipe=in $input.values.B1 \
        -print=column,ElementParameter,K1 \
        "-define=column,ParameterValue,BigB1 abs H / LEffective / K1Sign *" \
        -print=column,ElementType,[lindex $elementTypeList $index]

    # Define FSE_QUADRUPOLE as 0
    exec sddsprocess $input.values.B1  $input.values.FSEQ \
        -reprint=col,ElementParameter,FSE_QUADRUPOLE -redefine=col,ParameterValue,0

    # Combine 
    exec sddscombine $input.values.B0 $input.values.FSE $input.values.XKICK $input.values.B1 $input.values.FSEQ \
        -merge -pipe=out -retain=col,Element*,Parameter* \
        | sddsprocess -pipe=in $output "-redefine=col,ElementOccurence,1,type=long" 
    
    # Clean up
    APSDeleteTmpFileList -ID APSSetpointsToLatticeTGD
    
    return 1
}

proc APSLatticeToSetpoints {args} {
    set input ""
    set output ""
    set energyGeV 6.0
    APSStrictParseArguments {input output energyGeV}
    if ![string length $input] {
        return -code error "APSLatticeToSetpoints: provide input filename"
    }
    if ![string length $output] {
        return -code error "APSLatticeToSetpoints: provide output filename"
    }
    if ![file exists $input] {
        return -code error "APSLatticeToSetpoints: not found: $input"
    }
    if [file exists $output] {
        return -code error "APSLatticeToSetpoints: in use: $output"
    }
    file delete $output.qs
    if [catch  {APSLatticeToSetpointsQuadSext -input $input -output $output.qs -energyGeV $energyGeV} result] {
        return -code error "APSLatticeToSetpoints (1): $result"
    }
    if [file exists $output.qs] {
        set fileList $output.qs
    }
    foreach bend {Q4 Q5 Q8 M3 M4} {
        file delete $output.$bend
        APSAddToTmpFileList -ID APSLatticeToSetpoints -fileList $output.$bend
        if [catch {APSLatticeToSetpointsTGD -input $input -output $output.$bend -type $bend} result] {
            return -code error "APSLatticeToSetpoints (2): $result"
        }
        if {$result != 0} {
            lappend fileList $output.$bend
        }
    }
    if [llength $fileList]==0 {
        APSDeleteTmpFileList -ID APSLatticeToSetpoints
        return -code error "APSLatticeToSetpoints: nothing was selected it seems"
    }
    if [catch {eval exec sddscombine $fileList -merge -overwrite $output} result] {
        return -code error "APSLatticeToSetpoints (3): $result"
    }
    APSDeleteTmpFileList -ID APSLatticeToSetpoints
}

proc APSSetpointsToLattice {args} {
    set input ""
    set output ""
    set energyGeV 6.0
    APSStrictParseArguments {input output energyGeV}
    if ![string length $input] {
        return -code error "APSSetpointsToLattice: provide input filename"
    }
    if ![string length $output] {
        return -code error "APSSetpointsToLattice: provide output filename"
    }
    if ![file exists $input] {
        return -code error "APSSetpointsToLattice: not found: $input"
    }
    if [file exists $output] {
        return -code error "APSSetpointsToLattice: in use: $output"
    }
    file delete $output.qs
    if [catch {APSSetpointsToLatticeQuadSext -input $input -output $output.qs -energyGeV $energyGeV} result] {
        return -code error "APSSetpointsToLattice (1): $result"
    }
    set fileList ""
    if [file exists $output.qs] {
        set fileList $output.qs
    }
    foreach bend {Q4 Q5 Q8 M3 M4} {
        file delete $output.$bend
        APSAddToTmpFileList -ID APSSetpointsToLattice -fileList $output.$bend
        if [catch {APSSetpointsToLatticeTGD -input $input -output $output.$bend -type $bend} result] {
            return -code error "APSSetpointsToLattice (2): $result"
        }
        if {$result != 0} {
            lappend fileList $output.$bend
        }
    }
    if ![llength $fileList] {
        APSDeleteTmpFileList -ID APSSetpointsToLattice
        return -code error "APSSetpointsToLattice: nothing selected to convert"
    }
    if [catch {eval exec sddscombine $fileList -merge -overwrite $output} result] {
        APSDeleteTmpFileList -ID APSSetpointsToLattice
        return -code error "APSSetpointsToLattice: $result"
    }
    APSDeleteTmpFileList -ID APSSetpointsToLattice
}

proc APSUAdjustMagnetSetpoints {args} {
    # Modified from APS-era APsMpAdjustMagnetSetpoints
    set newSnapFile ""
    set referenceSnapFile ""
    set referenceSCRFile ""
    set adjustmentModel none
    set outputFileDir .
    set outputFileRoot ""
    set statusCallback ""
    set abortVariable ""

    if [APSStrictParseArguments {newSnapFile referenceSnapFile \
                                   referenceSCRFile adjustmentModel \
                                   outputFileDir outputFileRoot \
                                   statusCallback abortVariable}] {
        return -code error "APSUAdjustMagnetSetpoints: Invalid arguments."
    }

    if {$newSnapFile=="" || $referenceSnapFile=="" || \
          $referenceSCRFile=="" || $adjustmentModel=="" || \
          $outputFileDir=="" || $outputFileRoot=="" } {
        return -code error "APSUAdjustMagnetSetpoints: Invalid arguments."
    }
    if ![string compare $adjustmentModel none] {
        return
    }
    if [string length $statusCallback] {
        eval $statusCallback {"Processing $newSnapFile data with files $referenceSnapFile and $referenceSCRFile..."}
    }
    if ![file exists $newSnapFile] {
        return -code error "APSUAdjustMagnetSetpoints: Cannot find file $newSnapFile."
    }
    if ![file exists $referenceSnapFile] {
        return -code error "APSUAdjustMagnetSetpoints: Cannot find  file $referenceSnapFile."
    }
    if ![file exists $referenceSCRFile] {
        return -code error "APSUAdjustMagnetSetpoints: Cannot find  file $referenceSCRFile."
    }
    set tmproot /tmp/[APSTmpString]
    if {$adjustmentModel == "none"} {
        return
    }
    # Keep only magnet settings from SCR file, i.e. those control names
    # that are also contained in the reference snapshot file.
    if [catch {exec sddsselect $referenceSCRFile $referenceSnapFile -pipe=out \
                 -match=ControlName \
                 | sddsprocess -pipe=in ${tmproot}.SCR \
                 -scan=col,SCRSetpoint,ValueString,%lf \
             } result ] {
        return -code error "APSUAdjustMagnetSetpoints(0): $result"
    }
    if [catch {exec sddsprocess $referenceSnapFile ${tmproot}.ref \
                 -scan=col,RefSetpoint,ValueString,%lf \
             } result ] {
        return -code error "APSUAdjustMagnetSetpoints(1): $result"
    }
    switch $adjustmentModel {
        differenceInI {
            set operator1 -
            set operator2 +
        }
        ratioInI {
            set operator1 /
            set operator2 *
        }
    }
    # only adjust quadrupoles
    if [catch {exec sddsprocess $newSnapFile -pipe=out \
                 -scan=col,NewSetpoint,ValueString,%lf \
                 | sddsxref -pipe ${tmproot}.SCR \
                 -match=ControlName -take=SCRSetpoint \
                 | sddsxref -pipe ${tmproot}.ref \
                 -match=ControlName -take=RefSetpoint \
                 | sddsprocess -pipe \
                 "-def=col,IChange,NewSetpoint RefSetpoint $operator1,units=A" \
                 "-def=col,AdjustedSetPoint,SCRSetpoint IChange $operator2,units=A,description=I setpoint after doing $adjustmentModel adjustment model" \
                 | tee $outputFileDir/${outputFileRoot}.all.snp \
                 | sddsprocess -pipe=in $outputFileDir/${outputFileRoot}.Q.snp  \
                 -reprint=col,ValueString,%lf,AdjustedSetPoint \
             } result ] {
        return -code error "APSUAdjustMagnetSetpoints(2): $result"
    }
    if [catch {exec sddsprocess $outputFileDir/${outputFileRoot}.all.snp \
                 $outputFileDir/${outputFileRoot}.S.snp \
                 -match=col,ControlName=*:S* -noWarning \
             } result ] {
        return -code error "APSUAdjustMagnetSetpoints(2): $result"
    }
    if [catch {exec sddscombine $outputFileDir/${outputFileRoot}.Q.snp \
                 $outputFileDir/${outputFileRoot}.S.snp \
                 $outputFileDir/${outputFileRoot}.snp \
                 -merge -overWrite \
             } result ] {
        return -code error "APSUAdjustMagnetSetpoints(2): $result"
    }
    file delete -force $outputFileDir/${outputFileRoot}.Q.snp \
      $outputFileDir/${outputFileRoot}.S.snp

    if [string length $statusCallback] {
        eval $statusCallback "Done."
    }
    return
}

proc APSUAdjustParameterFile {args} {
    # Modified from APS-era APSMpAdjustParameterFile
    set referenceSCRFile ""
    set xrefFileList ""
    set energy 7
    set refParamFile ""
    set paramFile ""
    set BRhoPVName ""
    set outputParamFile ""
    set statusCallback ""
    if [APSStrictParseArguments {referenceSCRFile xrefFileList energy \
                                   refParamFile paramFile BRhoPVName \
                                   outputParamFile statusCallback}] {
        return -code error "APSUAdjustParameterFile: Invalid arguments."
    }
    if {$referenceSCRFile=="" || $xrefFileList=="" || $refParamFile=="" || \
          $paramFile=="" || $outputParamFile=="" } {
        return -code error "APSUAdjustParameterFile(1): Invalid arguments."
    }
    # convert I_ref to K1_ref, but only for quadrupoles.
    set tmproot [APSTmpString]
    set foundQxrefFile 0
    foreach xref $xrefFileList {
        #determine type of file from name
        if [regexp Q [file tail $xref]] {
            set foundQxrefFile 1
            if [catch {APSUMagnetKnLValues -SCRFile $referenceSCRFile \
                         -xrefFileList $xref \
                         -energy $energy  -BRhoPVName $BRhoPVName \
                         -outputFileDir /tmp \
                         -outputFileRoot ${tmproot} \
                         -statusCallback $statusCallback \
                     } result ] {
                return -code error "APSUAdjustParameterFile(2): $result"
            }
        }
    }
    # if there was no xref file with quadrupoles, then there
    # is no adjustments for this procedure to do.
    if !$foundQxrefFile {
        file copy -force $paramFile $outputParamFile
        if [string length $statusCallback] {
            eval $statusCallback {"Done. No change in parameter file needed."}
        }
        return  
    }
    # determine sum of K1(I_ref) + (K1 - K1_ref) for quads
    # file             column name
    # $tmpfile.KnL     Kn, which is K1(I_ref) above
    # $refParamFile    K1Reference
    # $paramFile       K1
    # Filter the parameter files for the quadrupole K1 ParameterName
    if [string length $statusCallback] {
        eval $statusCallback {"Calculating new K1's..."}
    }
    # note that the skew quads are inadvertedly included here, as
    # there is no easy way to filter them out.
    if [catch {exec sddsprocess $refParamFile /tmp/$tmproot.refK1 \
                 -match=col,ElementParameter=K1,ElementType=*QUAD*,& \
             } result ] {
        return -code error "APSUAdjustParameterFile(3): $result"
    }
    if [catch {exec sddsprocess $paramFile /tmp/$tmproot.K1 \
                 -match=col,ElementParameter=K1,ElementType=*QUAD*,& \
             } result ] {
        return -code error "APSUAdjustParameterFile(4): $result"
    }
    if [catch {exec sddsprocess $paramFile /tmp/$tmproot.other \
                 -match=col,ElementParameter=K1,ElementType=*QUAD*,&,! \
             } result ] {
        return -code error "APSUAdjustParameterFile(5): $result"
    }
    # Something to keep in mind: Since the KnL file /tmp/$tmproot.KnL 
    # doesn't include skew quads the final parameter file 
    # will not have skew quad K1's.
    if [catch {exec sddsxref /tmp/$tmproot.KnL /tmp/$tmproot.refK1  -pipe=out \
                   -match=ElementName -take=ParameterValue \
                 -rename=col,ParameterValue=K1Reference \
                 | sddsxref -pipe /tmp/$tmproot.K1 \
                 -match=ElementName -take=ParameterValue \
                 -rename=col,ParameterValue=K1 \
                 | sddsprocess -pipe=in /tmp/$tmproot.newK1 \
                 "-redef=col,ParameterValue,Kn K1 + K1Reference -" \
             } result ] {
        return -code error "APSUAdjustParameterFile(6): $result"
    }
    if [catch {exec sddscombine /tmp/$tmproot.newK1 /tmp/$tmproot.other \
                 $outputParamFile -overWrite -merge \
             } result ] {
        return -code error "APSUAdjustParameterFile(7): $result"
    }
    if [string length $statusCallback] {
        eval $statusCallback "Done."
    }
}

proc APSLatticeToSetpointsDeltaK1 {args} {
    set input ""
    set output ""
    set energyGeV 6.0
    set referenceParameters ""
    set referenceSnapshot ""
    set referenceEnergyGeV 6.0
    APSStrictParseArguments {input output energyGeV referenceParameters referenceSnapshot referenceEnergyGeV}

    if ![string length $input] {
        return -code error "APSLatticeToSetpointsDeltaK1: provide input filename"
    }
    if ![string length $output] {
        return -code error "APSLatticeToSetpointsDeltaK1: provide output filename"
    }
    if ![file exists $input] {
        return -code error "APSLatticeToSetpointsDeltaK1: not found: $input"
    }
    if [file exists $output] {
        return -code error "APSLatticeToSetpointsDeltaK1: in use: $output"
    }
    if ![string length $referenceParameters] {
        return -code error "APSLatticeToSetpointsDeltaK1: provide reference parameter filename"
    }
    if ![string length $referenceSnapshot] {
        return -code error "APSLatticeToSetpointsDeltaK1: provide reference snapshot filename"
    }
    if ![file exists $referenceParameters] {
        return -code error "APSLatticeToSetpointsDeltaK1: not found: $referenceParameters"
    }
    if ![file exists $referenceSnapshot] {
        return -code error "APSLatticeToSetpointsDeltaK1: not found: $referenceSnapshot"
    }
    
    set tmpRoot /tmp/[APSTmpString]
    APSAddToTmpFileList -ID APSLatticeToSetpointsDeltaK1 \
        -fileList [list $tmpRoot.K1Iref $tmpRoot.K1Eff $tmpRoot.others $tmpRoot.both $tmpRoot.K1Iref~ $tmpRoot.refparam]
    
    # Interpolate the reference currents to get nominal K1 values
    APSSetpointsToLattice -input $referenceSnapshot -output $tmpRoot.K1Iref -energyGeV $referenceEnergyGeV
    exec sddsprocess $tmpRoot.K1Iref -match=col,ElementParameter=K1 -nowarning
    
    exec sddsprocess $referenceParameters $tmpRoot.refparam -nowarning \
        "-match=col,ElementName=*M1*,ElementName=*M2*,|,ElementName=*SQ?,|,ElementName=*S?,|,!,ElementParameter=K1,&" 

    # Compute change in K1 from reference to new lattice
    # Then compute K1 value by offsetting from the K1(Iref) values
    exec sddsprocess $input -pipe=out -nowarning \
        "-match=col,ElementName=*M1*,ElementName=*M2*,|,ElementName=*SQ?,|,ElementName=*S?,|,!,ElementParameter=K1,&" \
        | sddsconvert -pipe -rename=col,ParameterValue=K1New \
        | sddsxref $tmpRoot.refparam -pipe -match=ElementName -take=ParameterValue -nowarning \
        -rename=col,ParameterValue=K1Ref \
        | sddsxref -pipe $tmpRoot.K1Iref -match=ElementName -take=ParameterValue -rename=col,ParameterValue=K1AtIRef -nowarning \
        | sddsprocess -pipe=in $tmpRoot.K1Eff "-redefine=col,ParameterValue,K1AtIRef K1New K1Ref - +" -nowarning

    exec sddsprocess $input $tmpRoot.others -nowarning \
        "-match=col,ElementName=*M1*,ElementName=*M2*,|,ElementName=*SQ?,|,ElementName=*S?,|,!,ElementParameter=K1,&,!" \

    exec sddscombine $tmpRoot.others $tmpRoot.K1Eff $tmpRoot.both -merge 
    
    APSLatticeToSetpoints -input $tmpRoot.both -output $output -energyGeV $energyGeV
    APSDeleteTmpFileList -ID APSLatticeToSetpointsDeltaK1
}

proc APSDeltaK1ToSetpoints {args} {
    set deltaK1 0.0
    set ElementName ""
    set output ""
    if [catch {APSStrictParseArguments {ElementName deltaK1 output}} result] {
        return -code error "APSDeltaK1ToSetpoints: $result"
    }
    if ![string length $ElementName] {
        return -code error "APSDeltaK1ToSetpoints: ElementName must not be blank"
    }
    eval set ElementGroup [os editstring Z: $ElementName]
    switch -glob $ElementName {
        *:Q\[45\] {
            set ElementType CCBEND
            set CNList $ElementName:PS:SetCurrentC,${ElementName}T:PS:SetCurrentC
        }
        *:Q8 -
        *:M\[34\] {
            set ElementType CSBEND
            set CNList $ElementName:PS:SetCurrentC,${ElementName}T:PS:SetCurrentC
        }
        *:Q? {
            set ElementType KQUAD
            set CNList $ElementName:PS:SetCurrentC
        }
        *:S? {
            return -code error "APSDeltaK1ToSetpoints: element $ElementName doesn't have K1 value"
        }
        default {
            return -code error "APSDeltaK1ToSetpoints: couldn't identify element type of $ElementName"
        }
    }
    set tmpRoot /tmp/[APSTmpString]
    APSAddToTmpFileList -ID APSDeltaK1ToSetpoints \
        -fileList [list $tmpRoot.req $tmpRoot.snap $tmpRoot.param $tmpRoot.param0 $tmpRoot.param2 $tmpRoot.snap2]

    # Find present setpoints
    if [catch {exec sddsmakedataset $tmpRoot.req -column=ControlName,type=string -data=[join $CNList ,]} result] {
        return -code error "APSDeltaK1ToSetpoints: $result"
    }
    if [catch {exec sddscasr -save  $tmpRoot.req $tmpRoot.snap -pendIoTime=10} result] {
        return -code error "APSDeltaK1ToSetpoints: $result"
    }

    # Convert to K1 etc.
    if [string compare $ElementType KQUAD]==0 {
        if [catch {APSSetpointsToLatticeQuadSext -input $tmpRoot.snap -output $tmpRoot.param} result] {
            return -code error "APSDeltaK1ToSetpoints: $result"
        }
    } else {
        if [catch {APSSetpointsToLatticeTGD -input $tmpRoot.snap -output $tmpRoot.param0 -type $ElementGroup} result] {
            return -code error "APSDeltaK1ToSetpoints: $result"
        }
        # Need a few extras from the reference lattice
        if [catch {exec sddsprocess /home/helios/oagData/sr/lattices/default/Basic.param $tmpRoot.extra \
                       -match=col,ElementName=$ElementName \
                       -match=col,ElementParameter=ANGLE,ElementParameter=L,|
            exec sddscombine $tmpRoot.param0  $tmpRoot.extra -merge $tmpRoot.param} result] {
            return -code error "APSDeltaK1ToSetpoints: $result"
        }
    }

    # Change the K1 value only
    if [catch {exec sddsprocess $tmpRoot.param $tmpRoot.param2 \
                   "-redefine=col,ParameterValue,ElementParameter \"K1\" streq ? $deltaK1 : 0 \$ ParameterValue +" } result] {
        return -code error "APSDeltaK1ToSetpoints: $result"
    }

    # Convert back to setpoints
    if [string compare $ElementType KQUAD]==0 {
        if [catch {APSLatticeToSetpointsQuadSext -input $tmpRoot.param2 -output $tmpRoot.snap2} result] {
            return -code error "APSDeltaK1ToSetpoints: $result"
        }
    } else {
        if [catch {APSLatticeToSetpointsTGD -input $tmpRoot.param2 -output $tmpRoot.snap2 -type $ElementGroup} result] {
            return -code error "APSDeltaK1ToSetpoints: $result"
        }
    }

    if [catch {sdds load $tmpRoot.snap2 data} result] {
        return -code error "APSDeltaK1ToSetpoints: $result"
    }

    foreach controlName [lindex $data(Column.ControlName) 0] value [lindex $data(Column.ValueString) 0] {
        lappend returnList $controlName=$value
    }
    if [llength $output] {
        file copy -force $tmpRoot.snap2 $output
    }
    APSDeleteTmpFileList -ID APSDeltaK1ToSetpoints
    return $returnList
}

proc BTSLatticeToSetpoints {args} {
    APSSetVarAndUpdate status "BTSLatticeToSetpoints $args"
    global apsBTSExcitationFileDirectory 
    set input ""
    set output ""
    set includeQuadrupoles 1
    set includeDipoles 1
    set energyGeV 6.0
    APSStrictParseArguments {input output includeQuadrupoles includeDipoles energyGeV}
    
    if ![string length $input] {
        return -code error "BTSLatticeToSetpoints: provide input filename"
    }
    if ![string length $output] {
        return -code error "BTSLatticeToSetpoints: provide output filename"
    }
    if ![file exists $input] {
        return -code error "BTSLatticeToSetpoints: not found: $input"
    }
    if [file exists $output] {
        return -code error "BTSLatticeToSetpoints: in use: $output"
    }

    set typeList {BEN QUAD}
    set BnLList {B0L B1L}
    set parameterList {ANGLE K1}
    set factorNameList {1 Length}
    set signNameList {AngleSign KnSign}
    set summaryFileList {BendSummary.sdds QuadSummary.sdds}
    set includeList [list $includeDipoles $includeQuadrupoles]
    set resultList ""

    exec sddsconvert $input $input.bare -delete=param,* -retain=column,ElementName,ElementType,ElementParameter,ParameterValue
    APSAddToTmpFileList -ID BTSLatticeToSetpoints -fileList $input.bare
    
    set typesDone 0
    foreach type $typeList BnL $BnLList parameter $parameterList factorName $factorNameList signName $signNameList summaryFile $summaryFileList include $includeList {
        if !$include continue
        if [catch {exec sddsprocess $input.bare -pipe=out -match=col,ElementType=*${type} -nowarning \
                       | sdds2stream -pipe -rows=bare} count] {
            return -code error "BTSLatticeToSetpoints (1): $count"
        }
        if $count==0 continue
        #puts stderr "$type $count"
        
        # Get the fractional strength errors
        APSAddToTmpFileList -ID BTSLatticeToSetpoints -fileList $input.FSE
        if [catch {exec sddsprocess $input.bare $input.FSE -match=col,ElementType=*${type} \
                       -match=col,ElementParameter=FSE -nowarning} result] {
            return -code error "BTSLatticeToSetpoints (2): $result"
        }
        #puts stderr [exec sddsprintout -col $input.FSE]

        # Get or compute the Kn values
        
        if [catch {exec sddsprocess $input.bare -pipe=out -match=col,ElementType=*${type} -match=col,ElementParameter=${parameter} \
                       -nowarning \
                       = Combine Kn values with the lengths and FSEs = \
                       | sddsxref $input.FSE -pipe -take=ParameterValue -rename=col,ParameterValue=FSE \
                       | sddsxref ${apsBTSExcitationFileDirectory}/$summaryFile -pipe -reuse=row \
                       -match=ElementName -take=*Sign,Length,PSName \
                       | sddsprocess -pipe \
                       "-define=parameter,E,$energyGeV,units=GeV" \
                       "-define=parameter,H,E 1e9 * c_mks /,units=T*m" \
                       "-define=col,StrengthParameter,ParameterValue,units=SI" \
                       "-define=col,KnLDesired,StrengthParameter $factorName * 1 FSE + *,units=SI" \
                       "-define=col,BnLDesired,KnLDesired H * $signName *,units=SI" \
                       -print=column,Filename,${apsBTSExcitationFileDirectory}/%s.excit,ElementName \
                       -print=column,ControlName,%s:PS:SetCurrentC,PSName \
                       | sddsinterpset -pipe \
                       -order=1 -data=fileColumn=Filename,interpolate=Current,functionOf=${BnL},column=BnLDesired \
                       | sddssort -pipe -column=PSName \
                       | sddsbreak -pipe -change=PSName \
                       | sddsprocess -pipe -process=ControlName,first,%s -process=Current,ave,%s \
                       | sddscollapse -pipe \
                       | sddsprocess -pipe=in $input.${type} -nowarning \
                       "-print=col,ValueString,%21.15le,Current"} result] {
            return -code error "BTSLatticeToSetpoints (3): $result"
        }
        APSAddToTmpFileList -ID BTSLatticeToSetpoints -fileList $input.$type
        
        file delete $input.FSE
        lappend resultList $input.${type}
        incr typesDone 1
    }
    
    if { $typesDone==0 } {
        # Not necessarily an error
        APSDeleteTmpFileList -ID BTSLatticeToSetpoints 
       return $typesDone
    }
    eval exec sddscombine $resultList -merge $output -retain=col,ControlName,ValueString,Current,Element*,Parameter*
    APSDeleteTmpFileList -ID BTSLatticeToSetpoints
    return $typesDone
}

proc BTSSetpointsToLattice {args} {
    global apsBTSExcitationFileDirectory
    set input ""
    set output ""
    set energyGeV 6.0
    APSStrictParseArguments {input output energyGeV}
    if ![string length $input] {
        return -code error "BTSSetpointsToLattice: provide input filename"
    }
    if ![string length $output] {
        return -code error "BTSSetpointsToLattice: provide output filename"
    }
    if ![file exists $input] {
        return -code error "BTSSetpointsToLattice: not found: $input"
    }
    if [file exists $output] {
        return -code error "BTSSetpointsToLattice: in use: $output"
    }

    set typeList {KQUAD CSBEN}
    set wildcardList {BTS:?Q? BTS:[ABC]B*} 
    set summaryFileList {QuadSummary.sdds BendSummary.sdds}
    set multList {1 0}

    set resultList ""
    
    foreach type $typeList wildcard $wildcardList mult $multList summaryFile $summaryFileList {
        #BTSSetVarAndUpdate status "BTSSetpointsToLattice $type"
        # Need only the PV name and value
        APSAddToTmpFileList -ID BTSSetpointsToLattice -fileList $output.$type.tmp1
        exec sddsconvert $input -pipe=out -retain=col,ControlName,ValueString \
            = Select only the PV names corresponding to the type of magnets = \
            | sddsprocess -pipe=in $output.$type.tmp1  "-match=col,ControlName=${wildcard}:PS:SetCurrentC" -nowarning \
            = Create the element name (used in the lattice) and scan the current = \
            -edit=column,PSName,ControlName,2S/:/100D \
            -scan=column,Current,ValueString,%le \
            = Create the name of the excitation file = \
            -print=column,Filename,${apsBTSExcitationFileDirectory}/%s.excit,ElementName,ElementName 
            
        if [catch {exec sdds2stream -rows=bare $output.$type.tmp1} count] {
            return -code error "BTSSetpointsToLattice: $count"
        }
        if { $count != 0} {
            #BTSSetVarAndUpdate status "BTSSetpointsToLattice $type $count"
            # Get length, sign for each element 
            APSAddToTmpFileList -ID BTSSetpointsToLattice -fileList $output.${type}
            exec sddsxref $output.$type.tmp1 ${apsBTSExcitationFileDirectory}/$summaryFile -pipe=out -reuse=row -nowarning \
                -match=ElementName -take=Length,KnSign \
                | sddsinterpset -pipe \
                = Interpolate all the files at the current values found in the input file = \
                -order=1 -data=fileColumn=Filename,interpolate=B${mult}L,functionOf=Current,column=Current \
                | sddsconvert -pipe -rename=col,B${mult}L=BnL \
                | sddsprocess -pipe=in $output.${type} -nowarning \
                = Convert BnL to Kn = \
                "-redefine=parameter,E,$energyGeV,units=GeV" \
                "-redefine=parameter,H,E 1e9 * c_mks /,units=T*m" \
                "-redefine=col,Bn,BnL KnSign * Length /,units=SI" \
                "-redefine=col,n,${mult},type=short" \
                "-redefine=col,ParameterValue,Bn H /,units=SI" \
                "-reprint=col,ElementParameter,K${mult}" \
                -reprint=col,ElementType,$type
            
            # Also set the FSE values to 0 in case these are set to something else in the lattice
            APSAddToTmpFileList -ID BTSSetpointsToLattice -fileList $output.${type}.FSE
            exec sddsconvert $output.${type} -pipe=out -retain=col,ElementName \
                | sddsprocess -pipe=in $output.${type}.FSE -nowarning \
                -reprint=col,ElementParameter,FSE -redefine=col,ParameterValue,0 \
                -reprint=col,ElementType,$type
            
            lappend resultList $output.$type $output.$type.FSE
        }
        file delete $output.$type.tmp1
    }
    #BTSSetVarAndUpdate status "$resultList"
    if [llength $resultList]==0 {
        APSDeleteTmpFileList -ID BTSSetpointsToLattice
        return 0
    }
    eval exec sddscombine $resultList -pipe=out -merge -retain=col,Element*,Parameter* \
        | sddsprocess -pipe=in $output "-redefine=col,ElementOccurence,1,type=long"
    
    APSDeleteTmpFileList -ID BTSSetpointsToLattice
    
    return 1
}
